#! /usr/bin/perl -w
################################################################################
# fan-upgradedb-centreon.pl                                                    #                                                                         
#                                                                              #
# Copyright (C) 2013 Olivier LI-KIANG-CHEONG                                   #
#                                                                              #
# This program is free software; you can redistribute it and/or modify         #
# it under the terms of the GNU General Public License as published by         #
# the Free Software Foundation; either version 2 of the License, or            #
# (at your option) any later version.                                          #
#                                                                              #
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY; without even the implied warranty of               #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                #
# GNU General Public License for more details.                                 #
#                                                                              #
# You should have received a copy of the GNU General Public License along      #
# with this program; if not, write to the Free Software Foundation, Inc.,      #
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.                  #
#                                                                              #
################################################################################
# Description : Configure distributed monitoring with Centreon and Nagios      #
# Including configuration database and adding poller                           #
################################################################################
# Version : 2.0                                                                #
# ChangeLog :                                                                  #
#  Update config centreon 2.3.X -> 2.4.X                                       #
#                                                                              #
################################################################################
# Author : Olivier LI-KIANG-CHEONG <lkco@gezen.fr>                             #
#                                                                              #
################################################################################

use strict;
use DBI;
use File::Copy;
use vars qw($mysql_user $mysql_passwd $mysql_host $mysql_database_oreon);
use vars qw($con);

my $NAME = "fan-upgradedb-centreon.pl";
my $LOG = "/var/log/centreon/fan-upgradedb-centreon.log";
my $CONF ="/etc/centreon/conf.pm";

# Include Configuration Data
require "$CONF";

sub writeLogFile($){
        open (LOG, ">> ".$LOG) || print "can't write $LOG: $!";
        print STDERR $_[0]."\n";
        print LOG localtime()." - ".$_[0]."\n";
        close LOG or warn $!;
}

sub CheckMySQLConnexion(){
   $con = DBI->connect("DBI:mysql:database=".$mysql_database_oreon.";host=".$mysql_host, $mysql_user, $mysql_passwd, {'RaiseError' => 0, 'PrintError' => 0, 'AutoCommit' => 1});
   if (!defined($con)) {
       writeLogFile("Warning when connecting to database : " . $DBI::errstr . "\n");
       return 0;
   } else{
       return 1;
   }
}

sub getCentreonVersion($){
    if (CheckMySQLConnexion()) {
        my $sth = $con->prepare("SELECT $_[0] FROM `informations` WHERE `key` = 'version' LIMIT 1;");
        if (!$sth->execute()) {
            writeLogFile("Error when getting centreon version : ".$sth->errstr);
        }
        my $data = $sth->fetchrow_hashref();
        return $data->{$_[0]};
    }
}

### MAIN ###
my $numArgs = $#ARGV + 1;
if ($numArgs != 3) {
    print "Usage: $NAME <newversion> <installdir> <state>\n";
    print "       $NAME 2.0.2 /usr/share/centreon/www/install $1\n";
    print "\n";
    print "If <newversion> is present in centreon database then <installdir> is moved in old\n";
    print "<state> equal $1\n";
    exit 1;
}

my $new_version = $ARGV[0];
my $install_dir = $ARGV[1];
my $state = $ARGV[2];

if ($state == 1 && !CheckMySQLConnexion()) {
    # First install and can't connect to centreon database
    rename($install_dir,$install_dir.".old") || writeLogFile ("Can't rename $install_dir"); 
    exit 0;
}

my $last_version = getCentreonVersion("value");
if ($new_version eq $last_version) {
    rename($install_dir,$install_dir.".old") || writeLogFile ("Can't rename $install_dir"); 
    exit 0;
}

## Update centreon 2.3.X -> 2.4.X
if (($new_version =~ "2.4\..*") and ($last_version =~ "2.3\..*")) {
    writeLogFile("[INFO] Update 2.3.X -> 2.4.X config files centreon"); 
    writeLogFile("[INFO] Backup /etc/centreon/conf.pm to /etc/centreon/conf.pm.bak"); 
    rename("/etc/centreon/conf.pm","/etc/centreon/conf.pm.bak");
    open(IN,"/etc/centreon/conf.pm.bak") || die ("Can't open /etc/centreon/conf.pm.bak");
    open (OUT,"> /etc/centreon/conf.pm") || die ("Can't open /etc/centreon/conf.pm");
    while (<IN>) {
        ( $_ !~ m/^1;.*/ ) ? print OUT $_ : next ;
    }
    close(IN);

    print OUT "\n";
    print OUT "# Central or Poller ?\n";
    print OUT "\$instance_mode = \"central\";\n";
    print OUT "\n";
    print OUT "# Centreon Centcore Command File\n";
    print OUT "\$cmdFile = \"/var/lib/centreon/centcore.cmd\";\n";
    print OUT "\n";
    print OUT "1;\n";
    close(OUT);

    if (-e "/etc/centreon/instCentCore.conf.rpmnew") {
        writeLogFile("[INFO] Backup /etc/centreon/instCentCore.conf to /etc/centreon/instCentCore.conf.bak"); 
        rename("/etc/centreon/instCentCore.conf","/etc/centreon/instCentCore.conf.bak");
        copy("/etc/centreon/instCentCore.conf.rpmnew","/etc/centreon/instCentCore.conf");
    }
    if (-e "/etc/centreon/instCentStorage.conf.rpmnew") {
        writeLogFile("[INFO] Backup /etc/centreon/instCentStorage.conf to /etc/centreon/instCentStorage.conf.bak"); 
        rename("/etc/centreon/instCentStorage.conf","/etc/centreon/instCentStorage.conf.bak");
        copy("/etc/centreon/instCentStorage.conf.rpmnew","/etc/centreon/instCentStorage.conf");
    }

    if (-e "/etc/centreon/instCentPlugins.conf.rpmnew") {
        writeLogFile("[INFO] Backup /etc/centreon/instCentPlugins.conf to /etc/centreon/instCentPlugins.conf.bak"); 
        rename("/etc/centreon/instCentPlugins.conf","/etc/centreon/instCentPlugins.conf.bak");
        copy("/etc/centreon/instCentPlugins.conf.rpmnew","/etc/centreon/instCentPlugins.conf");
    }

    if (-e "/etc/centreon/instCentWeb.conf.rpmnew") {
        writeLogFile("[INFO] Backup /etc/centreon/instCentWeb.conf to /etc/centreon/instCentWeb.conf.bak"); 
        rename("/etc/centreon/instCentWeb.conf","/etc/centreon/instCentWeb.conf.bak");
        copy("/etc/centreon/instCentWeb.conf.rpmnew","/etc/centreon/instCentWeb.conf.conf");
    }
}

